// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 1777 $
// $LastChangedDate:: 2019-12-03 16:10:30 +0100#$
//
// Script to load all X2C *.sci functions

mprintf("\n+ Loading X2C functions\n")

// get file separator
fs = filesep(); 

// get root directory of X2C installation
X2C_ROOT = fullpath(fullfile(getenv("X2C_Root")));

// standard (free) X2C Scilab script directory
scriptDirs = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "Scripts"];

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "Serial"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "PCAN"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "Protocol"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "Scope"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "Model"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// additional (commercial) X2C Scilab script directory
tmpDir = [X2C_ROOT + fs + "System" + fs + "Scilab" + fs + "x2c_wrapper" + fs + "Utils"];
if isdir(tmpDir) then
    scriptDirs = [scriptDirs, tmpDir];
end

// go trough script directorys
for tmpDir = scriptDirs
    // get files in script directory
    files = ls(tmpDir)';
    
    // go through file list
    for funcName = files
        if isfile([tmpDir + fs + funcName]) then
            if ~strcmp(part(funcName, $-3:$), ".sci", "i") then
                // load function if funcName ends with *.sci
                exec([tmpDir + fs + funcName], -1);
            end
        end
    end
end



// clean up
clear("fs", "X2C_ROOT", "scriptDirs", "tmpDir", "files", "funcName")

mprintf("- Done\n");
